#!/usr//bin/env sage
from sage.all import *
from optparse import OptionParser

def bench_elimination(m,n):
    sage_mul = [[],[]]
    magm_mul = [[],[]]
    sage_elm = [[],[]]
    magm_elm = [[],[]]
    for i in range(2,10+1):
        K,a = GF(2**i,'a').objgen()    
        A = random_matrix(K, m, n)
        B = random_matrix(K, m, n)
        t = walltime()
        C = A*B
        t = walltime(t)
        print "sage mul: %2d: %5.2f"%(i, t),
        sage_mul.append(t)

        _ = magma.eval("K<a> := GF(2^%d)"%i)
        A = magma("RandomMatrix(K,%d,%d)"%(m,n))
        B = magma("RandomMatrix(K,%d,%d)"%(m,n))
        t = magma.cputime()
        C = A*B
        t = magma.cputime(t)
        print "magm mul: %2d: %5.2f"%(i, t),
        magm_mul.append(t)

        A = random_matrix(K, m, n)
        t = walltime()
        E = A.echelonize('heuristic')
        t = walltime(t)
        print "sage elm: %2d: %5.2f"%(i, t),
        sage_elm.append(t)

        _ = magma.eval("K<a> := GF(2^%d)"%i)
        A = magma("RandomMatrix(K,%d,%d)"%(m,n))
        B = magma("RandomMatrix(K,%d,%d)"%(m,n))
        t = magma.cputime()
        C = A.EchelonForm()
        t = magma.cputime(t)
        print "magm elm: %2d: %5.2f"%(i, t),
        magm_elm.append(t)

        print

    print "'''%d x %d'''"%(m,n)
    print "||     n || M4rie mul || Magma mul || M4rie elim || Magma elim ||"
    for i in range(2,11):
        print "||    %2d ||     %5.2f ||     %5.2f ||      %5.2f ||      %5.2f ||"%(i,sage_mul[i],magm_mul[i],sage_elm[i],magm_elm[i],)

if __name__ == '__main__':
    parser = OptionParser(usage="""\
Time M4RIE.

Usage: %prog [options]
""")
    parser.add_option('-m', type='int', action='store', help='Number of Rows')
    parser.add_option('-n', type='int', action='store', help='Number of Columns')
    opts, args = parser.parse_args()

    if not opts.n:
        opts.n = opts.m

    if not opts.m:
        opts.m = opts.n

    bench_elimination(opts.m,opts.n)
        
